require 'puppet/face'
require 'digest/md5'
require 'fileutils'
require 'json'
require 'yaml'
require 'shellwords'
require 'open3'

Puppet::Face.define(:local, '0.0.1') do

  action :install do

    when_invoked do |hiera_source_path, options|



      # Hiera source
      raise Puppet::Error, '`' + hiera_source_path + '` does not exist' unless File.exists? hiera_source_path
      hiera_source_path = File.absolute_path hiera_source_path
      hiera_source = File.new(hiera_source_path)
      local_key = Digest::MD5.hexdigest(hiera_source_path)
      local_dir = `puppet apply --configprint vardir`.strip + '/puppet-local/' + local_key + '/'
      FileUtils.mkdir_p local_dir unless Dir.exists? local_dir

      # Hiera config
      hiera_config_path = local_dir + 'hiera.yaml'
      hiera_config = {
          :backends => ['json'],
          :hierarchy => [File.basename(hiera_source_path).sub(/\.json$/, '')],
          :json => {:datadir => File.dirname(hiera_source)}
      }
      File.new(hiera_config_path, 'w').puts(hiera_config.to_yaml.gsub('!ruby/sym ', ':'))

      # Modules
      modules_paths = []
      json = JSON.parse hiera_source.read
      json['git-modules'].each do |source, version|
        modules_path = local_dir + Digest::MD5.hexdigest(source)
        system('git clone ' + source + ' ' + modules_path) unless Dir.exists? modules_path
        Dir.chdir modules_path
        system('git checkout ' + version)
        modules_paths.push modules_path + '/modules'
      end

      command = 'puppet apply -e \'include hiera("classes")\' --hiera_config ' + hiera_config_path.shellescape + ' --modulepath ' + modules_paths.join(':')
      Open3.popen2(command) do |input, output, wait_thr|
        output.read
      end
    end

    when_rendering :console do |output, hiera_file, options|
      output
    end
  end
end